'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global browser, punycode */
/* exported mayOpenDialogUI, rightclickedItem, YT_LISTENER_ADDED */

// Global const to hold thirty minutes in milliseconds
const THIRTY_MINUTES_IN_MILLISECONDS = 1800000;

// Set to true to get noisier console.log statements
const VERBOSE_DEBUG = false;

// Global variable to track YouTube listener
let YT_LISTENER_ADDED = false;

if (`${window.location.origin}/` === browser.extension.getURL('')) {
  // above line avoids content scripts making their host page break frames
  if (window.top !== window) {
    window.location.replace('about:blank');
  }
}

// Global variable to track whether a ui dialog wizard can be opened
let mayOpenDialogUI = true;

// Global variable to track the item that's right-clicked
let rightclickedItem = null;

// Run a function on the background page.
// Inputs (positional):
//   first, a string - the name of the function to call
//   then, any arguments to pass to the function (optional)
//   then, a callback:function(return_value:any) (optional)
const BGcall = function (...args) {
  const newArgs = [];
  for (let i = 0; i < args.length; i += 1) {
    newArgs.push(args[i]);
  }
  const fn = newArgs.shift();
  const hasCallback = (typeof newArgs[newArgs.length - 1] === 'function');
  const callback = (hasCallback ? newArgs.pop() : function emptyCallback() {
  });

  if (!hasCallback) {
    browser.runtime.sendMessage({ command: 'call', fn, args: newArgs });
  } else {
    browser.runtime.sendMessage({ command: 'call', fn, args: newArgs }, callback);
  }
};

let loggingEnable = false;
let log = function emptyLogFunction() { };

// Enabled in adblock_start_common.js and background.js if the user wants
const logging = function (enabled) {
  if (enabled) {
    loggingEnable = true;
    log = function logFunction(...args) {
      // comment out for verbosity
      if (VERBOSE_DEBUG || args[0] !== '[DEBUG]') {
        /* eslint-disable-next-line no-console */
        console.log(...args);
      }
    };
  } else {
    loggingEnable = false;
  }
};

Object.assign(window, { log });
logging(false); // disabled by default

// Behaves very similarly to $.ready() but does not require jQuery.
const onReady = function (callback) {
  if (document.readyState === 'complete') {
    window.setTimeout(callback, 0);
  } else {
    window.addEventListener('load', callback, false);
  }
};

const translate = function (messageID, args) {
  let newArgs = args;
  if (Array.isArray(newArgs)) {
    for (let i = 0; i < newArgs.length; i += 1) {
      if (typeof newArgs[i] !== 'string') {
        newArgs[i] = newArgs[i].toString();
      }
    }
  } else if (newArgs && typeof newArgs !== 'string') {
    newArgs = newArgs.toString();
  }
  if (!messageID || typeof messageID !== 'string') {
    // eslint-disable-next-line no-console
    console.trace('translate called with empty message id', messageID, newArgs);
  }
  return browser.i18n.getMessage(messageID, newArgs);
};

const splitMessageWithReplacementText = function (rawMessageText, messageID) {
  const anchorStartPos = rawMessageText.indexOf('[[');
  const anchorEndPos = rawMessageText.indexOf(']]');

  if (anchorStartPos === -1 || anchorEndPos === -1) {
    log('replacement tag not found', messageID, rawMessageText, anchorStartPos, anchorEndPos);
    return { error: 'no brackets found' };
  }
  const returnObj = {};
  returnObj.anchorPrefixText = rawMessageText.substring(0, anchorStartPos);
  returnObj.anchorText = rawMessageText.substring(anchorStartPos + 2, anchorEndPos);
  returnObj.anchorPostfixText = rawMessageText.substring(anchorEndPos + 2);
  return returnObj;
};

// Determine what language the user's browser is set to use
const determineUserLanguage = function () {
  if (browser && browser.i18n && browser.i18n.getUILanguage) {
    return browser.i18n.getUILanguage().match(/^[a-z]+/i)[0];
  }
  return null;
};

const localizePage = function () {
  // translate a page into the users language
  $('[i18n]:not(.i18n-replaced, [i18n_replacement_el])').each(function translateI18n() {
    $(this).text(translate($(this).attr('i18n')));
  });

  $('[i18n_value]:not(.i18n-replaced)').each(function translateValue() {
    $(this).val(translate($(this).attr('i18n_value')));
  });

  $('[i18n_title]:not(.i18n-replaced)').each(function translateTitle() {
    $(this).attr('title', translate($(this).attr('i18n_title')));
  });

  $('[i18n_placeholder]:not(.i18n-replaced)').each(function translatePlaceholder() {
    $(this).attr('placeholder', translate($(this).attr('i18n_placeholder')));
  });

  $('[i18n_replacement_el]:not(.i18n-replaced)').each(function replaceDummy() {
    // Replace a dummy <a/> inside of localized text with a real element.
    // Give the real element the same text as the dummy link.
    const messageID = $(this).attr('i18n');
    if (!messageID || typeof messageID !== 'string') {
      $(this).addClass('i18n-replaced');
      return;
    }
    if (!$(this).get(0).firstChild) {
      log('returning, no first child found', $(this).attr('i18n'));
      return;
    }
    if (!$(this).get(0).lastChild) {
      log('returning, no last child found', $(this).attr('i18n'));
      return;
    }
    const replaceElId = `#${$(this).attr('i18n_replacement_el')}`;
    if ($(replaceElId).length === 0) {
      log('returning, no child element found', $(this).attr('i18n'), replaceElId);
      return;
    }
    const rawMessageText = browser.i18n.getMessage(messageID) || '';
    const messageSplit = splitMessageWithReplacementText(rawMessageText, messageID);
    $(this).get(0).firstChild.nodeValue = messageSplit.anchorPrefixText;
    $(this).get(0).lastChild.nodeValue = messageSplit.anchorPostfixText;
    if ($(replaceElId).get(0).tagName === 'INPUT') {
      $(`#${$(this).attr('i18n_replacement_el')}`).prop('value', messageSplit.anchorText);
    } else {
      $(`#${$(this).attr('i18n_replacement_el')}`).text(messageSplit.anchorText);
    }

    // If localizePage is run again, don't let the [i18n] code above
    // clobber our work
    $(this).addClass('i18n-replaced');
  });

  // Make a right-to-left translation for Arabic and Hebrew languages
  const language = determineUserLanguage();
  if (language === 'ar' || language === 'he') {
    $('#main_nav').removeClass('right').addClass('left');
    $('.adblock-logo').removeClass('left').addClass('right');
    $('.closelegend').css('float', 'left');
    document.documentElement.dir = 'rtl';
  }
};

// Parse a URL. Based upon http://blog.stevenlevithan.com/archives/parseuri
// parseUri 1.2.2, (c) Steven Levithan <stevenlevithan.com>, MIT License
// Inputs: url: the URL you want to parse
// Outputs: object containing all parts of |url| as attributes
const parseUriRegEx = /^(([^:]+(?::|$))(?:(?:\w+:)?\/\/)?(?:[^:@/]*(?::[^:@/]*)?@)?(([^:/?#]*)(?::(\d*))?))((?:[^?#/]*\/)*[^?#]*)(\?[^#]*)?(#.*)?/;
const parseUri = function (url) {
  const matches = parseUriRegEx.exec(url);

  // The key values are identical to the JS location object values for that key
  const keys = ['href', 'origin', 'protocol', 'host', 'hostname', 'port',
    'pathname', 'search', 'hash'];
  const uri = {};
  for (let i = 0; (matches && i < keys.length); i += 1) {
    uri[keys[i]] = matches[i] || '';
  }
  return uri;
};

// Parses the search part of a URL into an key: value object.
// e.g., ?hello=world&ext=adblock would become {hello:"world", ext:"adblock"}
// Inputs: search: the search query of a URL. Must have &-separated values.
parseUri.parseSearch = function parseUriSearch(search) {
  // Fails if a key exists twice (e.g., ?a=foo&a=bar would return {a:"bar"}
  const searchArray = search.substring(search.indexOf('?') + 1).split('&');
  const params = {};
  let pair;
  for (let i = 0; i < searchArray.length; i += 1) {
    pair = searchArray[i].split('=');
    if (pair[0] && !pair[1]) {
      pair[1] = '';
    }
    if (params[decodeURIComponent(pair[0])] || decodeURIComponent(pair[1]) !== 'undefined') {
      params[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1]);
    }
  }

  return params;
};

// Strip third+ level domain names from the domain and return the result.
// Inputs: domain: the domain that should be parsed
//         keepDot: true if trailing dots should be preserved in the domain
// Returns: the parsed domain
parseUri.secondLevelDomainOnly = function stripThirdDomain(domain, keepDot) {
  const match = domain.match(/([^.]+\.(?:co\.)?[^.]+)\.?$/) || [domain, domain];
  return match[keepDot ? 0 : 1].toLowerCase();
};

// Return |domain| encoded in Unicode
const getUnicodeDomain = function (domain) {
  if (domain) {
    return punycode.toUnicode(domain);
  }
  return domain;
};

// Return |url| encoded in Unicode
const getUnicodeUrl = function (url) {
  // URLs encoded in Punycode contain xn-- prefix
  if (url && url.indexOf('xn--') > 0) {
    const parsed = parseUri(url);
    // IDN domains have just hostnames encoded in punycode
    parsed.href = parsed.href.replace(parsed.hostname, punycode.toUnicode(parsed.hostname));
    return parsed.href;
  }

  return url;
};

// Return obj[value], first setting it to |defaultValue| if it is undefined.
const setDefault = function (obj, value, defaultValue) {
  if (obj[value] === undefined) {
    /* eslint-disable-next-line no-param-reassign */
    obj[value] = defaultValue;
  }
  return obj[value];
};

// Inputs: key:string.
// Returns value if key exists, else undefined.
const storageGet = function (key) {
  const store = localStorage;
  if (store === undefined) {
    return undefined;
  }

  const json = store.getItem(key);
  if (json == null) {
    return undefined;
  }
  try {
    return JSON.parse(json);
  } catch (e) {
    log(`Couldn't parse json for ${key}`);
    return undefined;
  }
};

// Inputs: key:string, value:object.
// If value === undefined, removes key from storage.
// Returns undefined.
const storageSet = function (key, value) {
  const store = localStorage;
  if (value === undefined) {
    store.removeItem(key);
    return;
  }

  try {
    store.setItem(key, JSON.stringify(value));
  } catch (ex) {
    // ToDo - handle errors gracefully.
  }
};

// Sets expirable object in storage to be used in place of a cookie
// Inputs:
//   name: string,
//   value: object,
//   millisecondsUntilExpire: number of milliseconds until the "cookie" expires
const setStorageCookie = function (name, value, millisecondsUntilExpire) {
  const expirationTime = Date.now() + (millisecondsUntilExpire || 0);

  storageSet(name, { value, expires: expirationTime });
};

// Returns value of storage "cookie" or undefined if the it doesn't exist or
// has expired
// Inputs:
//  name: string
const getStorageCookie = function (name) {
  const storedCookie = storageGet(name);

  if (storedCookie && (storedCookie.expires > Date.now())) {
    return storedCookie.value;
  }

  return undefined;
};
