'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global browser, DOM, ThemeManager, BGcall */

// ThemeUI implements UI controls and previews for a Theme.
class ThemeUI {
  constructor(theme, group) {
    this.theme = theme;
    const themeID = `preview-${this.theme.kind}-theme-${this.theme.themeName}`;
    group.add(this);

    this.$box = DOM.div([this.theme.kind, 'theme-box'],
      DOM.div('theme-box-content', [
        DOM.div('theme-header', this.theme.header
          ? [
            DOM.img(`/themes/${this.theme.themeName}/minicard.svg`),
          ]
          : [
            DOM.p(this.theme.body),
          ]),
        DOM.div('theme-footer', DOM.p(this.theme.name)),
      ], { ready: $content => this.theme.apply($content) }));
    DOM.addClassIf(this.$box, this.theme.unlocked, 'unlocked');
    DOM.addClassIf(this.$box, this.theme.selected, 'selected');

    // Add overlays to preview and activate for themes that are enabled and unlocked
    if (this.theme.enabled && this.theme.unlocked) {
      this.theme.apply(this.$box);
      this.$box.append(
        DOM.div('theme-hover-overlay', [
          DOM.div('theme-hover-overlay-header', [
            DOM.a(DOM.icon('search', 'md-72'), `#${themeID}`),
          ]),
          DOM.div('theme-hover-overlay-footer', [
            DOM.a('apply_theme', () => {
              group.select(this.theme.themeName);
            }),
          ]),
        ]),
        DOM.div('theme-preview-overlay', [
          DOM.img(`/themes/${this.theme.themeName}/preview_${this.theme.kind}.svg`, 'preview'),
          DOM.a(DOM.icon(['lens', 'cancel'], 'md-24', ['md-stack-1x', 'md-stack-1x icon'])),
        ], {
          id: themeID,
          click: () => {
            window.location.href = '#';
          },
        }),
      );
    }
  }

  // thumbnail returns a div element that showcases, previews, and
  // activates the theme.
  thumbnail() {
    return this.$box;
  }

  // selected sets the current selected state of this theme.
  selected(isSelected) {
    if (isSelected) {
      this.$box.addClass('selected');
      this.theme.select();
      if (this.theme.kind === 'options') {
        this.theme.apply();
      }
    } else {
      this.$box.removeClass('selected');
    }
  }
}

// ThemeUIGroup contains all the theme UIs that form a group.
// Currently used to ensure selected theme properly updates other themes in the group.
class ThemeUIGroup {
  constructor(settings) {
    this.themes = [];
    this.settings = settings || {};
  }

  add(theme) {
    this.themes.push(theme);
  }

  select(name) {
    for (let i = 0; i < this.themes.length; i++) {
      const themeUI = this.themes[i];
      const { themeName } = themeUI.theme; // yikes
      const isSelected = themeName === name;
      if (isSelected) {
        themeUI.selected(true);
        switch (themeUI.theme.kind) {
          case 'popup':
            this.settings.popup_menu = themeName;
            break;
          case 'options':
            this.settings.options_page = themeName;
            break;
          default:
        }
      } else {
        themeUI.selected(false);
      }
      themeUI.selected(isSelected);
    }
    BGcall('setSetting', 'color_themes', this.settings);
  }
}

// kinds of themes that we support.
const kinds = [
  { kind: 'popup', target: 'popup-menu-themes' },
  { kind: 'options', target: 'options-page-themes' },
];

// themeUIs grouped by kind
const groups = {};

// setup the themes for each kind of UI that accepts a theme.
function setup(settings, config) {
  const $wrapper = $(`#${config.target} .theme-group`);
  const themes = new ThemeManager(config.kind, config.selected);
  const group = new ThemeUIGroup(settings);
  groups[config.kind] = group;
  themes.unlocked.forEach((theme) => {
    const ui = new ThemeUI(theme, group);
    $wrapper.append(ui.thumbnail());
  });
}

$(document).ready(() => {
  const settings = browser.extension.getBackgroundPage().getSettings().color_themes || {};
  kinds.forEach((kind) => {
    const config = {
      target: kind.target,
      kind: kind.kind,
    };
    switch (kind.kind) {
      case 'popup':
        config.selected = settings.popup_menu || 'default';
        break;
      case 'options':
        config.selected = settings.options_page || 'default';
        break;
      default:
    }
    setup(settings, config);
  });
});
